@@ -17,8 +17,9 @@ import {
   TypedVariableModel,
   UrlQueryValue,
 } from '@grafana/data';
+import { PromQuery } from '@grafana/prometheus';
 import { RefreshEvent, TimeRangeUpdatedEvent, config } from '@grafana/runtime';
-import { Dashboard, DashboardLink } from '@grafana/schema';
+import { Dashboard, DashboardLink, VariableModel } from '@grafana/schema';
 import { DEFAULT_ANNOTATION_COLOR } from '@grafana/ui';
 import { GRID_CELL_HEIGHT, GRID_CELL_VMARGIN, GRID_COLUMN_COUNT, REPEAT_DIR_VERTICAL } from 'app/core/constants';
 import { contextSrv } from 'app/core/services/context_srv';
@@ -27,8 +28,7 @@ import { isAngularDatasourcePluginAndNotHidden } from 'app/features/plugins/angu
 import { variableAdapters } from 'app/features/variables/adapters';
 import { onTimeRangeUpdated } from 'app/features/variables/state/actions';
 import { GetVariables, getVariablesByKey } from 'app/features/variables/state/selectors';
-import { PromQuery } from 'app/plugins/datasource/prometheus/types';
-import { CoreEvents, DashboardMeta, KioskMode } from 'app/types';
+import { CoreEvents, DashboardMeta } from 'app/types';
 import { DashboardMetaChangedEvent, DashboardPanelsChangedEvent, RenderEvent } from 'app/types/events';
 
 import { appEvents } from '../../../core/core';
@@ -44,7 +44,7 @@ import { getTimeSrv } from '../services/TimeSrv';
 import { mergePanels, PanelMergeInfo } from '../utils/panelMerge';
 
 import { DashboardMigrator } from './DashboardMigrator';
-import { PanelModel } from './PanelModel';
+import { explicitlyControlledMigrationPanels, PanelModel } from './PanelModel';
 import { TimeModel } from './TimeModel';
 import { deleteScopeVars, isOnTheSameGridRow } from './utils';
 
@@ -56,6 +56,12 @@ export interface CloneOptions {
 
 export type DashboardLinkType = 'link' | 'dashboards';
 
+/** @experimental */
+export interface ScopeMeta {
+  trait: string;
+  groups: string[];
+}
+
 export class DashboardModel implements TimeModel {
   /** @deprecated use UID */
   id: any;
@@ -87,10 +93,11 @@ export class DashboardModel implements TimeModel {
   panelInEdit?: PanelModel;
   panelInView?: PanelModel;
   fiscalYearStartMonth?: number;
+  scopeMeta?: ScopeMeta;
   private panelsAffectedByVariableChange: number[] | null;
   private appEventsSubscription: Subscription;
   private lastRefresh: number;
-  private timeRangeUpdatedDuringEdit = false;
+  private timeRangeUpdatedDuringEditOrView = false;
   private originalDashboard: Dashboard | null = null;
 
   // ------------------
@@ -117,7 +124,7 @@ export class DashboardModel implements TimeModel {
     appEventsSubscription: true,
     panelsAffectedByVariableChange: true,
     lastRefresh: true,
-    timeRangeUpdatedDuringEdit: true,
+    timeRangeUpdatedDuringEditOrView: true,
     originalDashboard: true,
   };
 
@@ -127,9 +134,6 @@ export class DashboardModel implements TimeModel {
     options?: {
       // By default this uses variables from redux state
       getVariablesFromState?: GetVariables;
-
-      // Force the loader to migrate panels
-      autoMigrateOldPanels?: boolean;
     }
   ) {
     this.getVariablesFromState = options?.getVariablesFromState ?? getVariablesByKey;
@@ -148,7 +152,7 @@ export class DashboardModel implements TimeModel {
     this.time = data.time ?? { from: 'now-6h', to: 'now' };
     this.timepicker = data.timepicker ?? {};
     this.liveNow = data.liveNow;
-    this.templating = this.ensureListExist(data.templating);
+    this.templating = this.removeNullValuesFromVariables(this.ensureListExist(data.templating));
     this.annotations = this.ensureListExist(data.annotations);
     this.refresh = data.refresh;
     this.snapshot = data.snapshot;
@@ -158,6 +162,8 @@ export class DashboardModel implements TimeModel {
     this.links = data.links ?? [];
     this.gnetId = data.gnetId || null;
     this.panels = map(data.panels ?? [], (panelData) => new PanelModel(panelData));
+    // @ts-expect-error - experimental and it's not included in the schema
+    this.scopeMeta = data.scopeMeta;
     // Deep clone original dashboard to avoid mutations by object reference
     this.originalDashboard = cloneDeep(data);
     this.originalTemplating = cloneDeep(this.templating);
@@ -169,59 +175,6 @@ export class DashboardModel implements TimeModel {
     this.initMeta(meta);
     this.updateSchema(data);
 
-    // Auto-migrate old angular panels
-    const shouldMigrateAllAngularPanels =
-      options?.autoMigrateOldPanels || !config.angularSupportEnabled || config.featureToggles.autoMigrateOldPanels;
-
-    const shouldMigrateExplicitAngularPanels =
-      config.featureToggles.autoMigrateGraphPanel ||
-      config.featureToggles.autoMigrateTablePanel ||
-      config.featureToggles.autoMigratePiechartPanel ||
-      config.featureToggles.autoMigrateWorldmapPanel ||
-      config.featureToggles.autoMigrateStatPanel;
-
-    // Handles both granular and all angular panel migration
-    if (shouldMigrateAllAngularPanels || shouldMigrateExplicitAngularPanels) {
-      for (const panel of this.panelIterator()) {
-        if (
-          !panel.autoMigrateFrom &&
-          panel.type === 'graph' &&
-          (config.featureToggles.autoMigrateGraphPanel || shouldMigrateAllAngularPanels)
-        ) {
-          panel.autoMigrateFrom = panel.type;
-          panel.type = 'timeseries';
-        } else if (
-          !panel.autoMigrateFrom &&
-          panel.type === 'table-old' &&
-          (config.featureToggles.autoMigrateTablePanel || shouldMigrateAllAngularPanels)
-        ) {
-          panel.autoMigrateFrom = panel.type;
-          panel.type = 'table';
-        } else if (
-          !panel.autoMigrateFrom &&
-          panel.type === 'grafana-piechart-panel' &&
-          (config.featureToggles.autoMigratePiechartPanel || shouldMigrateAllAngularPanels)
-        ) {
-          panel.autoMigrateFrom = panel.type;
-          panel.type = 'piechart';
-        } else if (
-          !panel.autoMigrateFrom &&
-          panel.type === 'grafana-worldmap-panel' &&
-          (config.featureToggles.autoMigrateWorldmapPanel || shouldMigrateAllAngularPanels)
-        ) {
-          panel.autoMigrateFrom = panel.type;
-          panel.type = 'geomap';
-        } else if (
-          !panel.autoMigrateFrom &&
-          (panel.type === 'singlestat' || panel.type === 'grafana-singlestat-panel') &&
-          (config.featureToggles.autoMigrateStatPanel || shouldMigrateAllAngularPanels)
-        ) {
-          panel.autoMigrateFrom = panel.type;
-          panel.type = 'stat';
-        }
-      }
-    }
-
     this.addBuiltInAnnotationQuery();
     this.sortPanelsByGridPos();
     this.panelsAffectedByVariableChange = null;
@@ -333,6 +286,8 @@ export class DashboardModel implements TimeModel {
    *
    * @internal and experimental
    */
+  // TODO: remove this as it's not being used anymore
+  // Also remove public/app/features/dashboard/utils/panelMerge.ts
   updatePanels(panels: IPanelModel[]): PanelMergeInfo {
     const info = mergePanels(this.panels, panels ?? []);
     if (info.changed) {
@@ -426,8 +381,8 @@ export class DashboardModel implements TimeModel {
     this.events.publish(new TimeRangeUpdatedEvent(timeRange));
     dispatch(onTimeRangeUpdated(this.uid, timeRange));
 
-    if (this.panelInEdit) {
-      this.timeRangeUpdatedDuringEdit = true;
+    if (this.panelInEdit || this.panelInView) {
+      this.timeRangeUpdatedDuringEditOrView = true;
     }
   }
 
@@ -480,7 +435,7 @@ export class DashboardModel implements TimeModel {
   initEditPanel(sourcePanel: PanelModel): PanelModel {
     getTimeSrv().stopAutoRefresh();
     this.panelInEdit = sourcePanel.getEditClone();
-    this.timeRangeUpdatedDuringEdit = false;
+    this.timeRangeUpdatedDuringEditOrView = false;
     return this.panelInEdit;
   }
 
@@ -490,34 +445,30 @@ export class DashboardModel implements TimeModel {
 
     getTimeSrv().resumeAutoRefresh();
 
-    if (this.panelsAffectedByVariableChange || this.timeRangeUpdatedDuringEdit) {
-      this.startRefresh({
-        panelIds: this.panelsAffectedByVariableChange ?? [],
-        refreshAll: this.timeRangeUpdatedDuringEdit,
-      });
-      this.panelsAffectedByVariableChange = null;
-      this.timeRangeUpdatedDuringEdit = false;
-    }
+    this.refreshIfPanelsAffectedByVariableChangeOrTimeRangeChanged();
   }
 
   initViewPanel(panel: PanelModel) {
     this.panelInView = panel;
+    this.timeRangeUpdatedDuringEditOrView = false;
     panel.setIsViewing(true);
   }
 
   exitViewPanel(panel: PanelModel) {
     this.panelInView = undefined;
     panel.setIsViewing(false);
-    this.refreshIfPanelsAffectedByVariableChange();
+    this.refreshIfPanelsAffectedByVariableChangeOrTimeRangeChanged();
   }
 
-  private refreshIfPanelsAffectedByVariableChange() {
-    if (!this.panelsAffectedByVariableChange) {
-      return;
+  private refreshIfPanelsAffectedByVariableChangeOrTimeRangeChanged() {
+    if (this.panelsAffectedByVariableChange || this.timeRangeUpdatedDuringEditOrView) {
+      this.startRefresh({
+        panelIds: this.panelsAffectedByVariableChange ?? [],
+        refreshAll: this.timeRangeUpdatedDuringEditOrView,
+      });
+      this.panelsAffectedByVariableChange = null;
+      this.timeRangeUpdatedDuringEditOrView = false;
     }
-
-    this.startRefresh({ panelIds: this.panelsAffectedByVariableChange, refreshAll: false });
-    this.panelsAffectedByVariableChange = null;
   }
 
   private ensurePanelsHaveUniqueIds() {
@@ -531,6 +482,28 @@ export class DashboardModel implements TimeModel {
     }
   }
 
+  private removeNullValuesFromVariables(templating: { list: VariableModel[] }) {
+    if (!templating.list.length) {
+      return templating;
+    }
+
+    for (const variable of templating.list) {
+      if (variable.current) {
+        // this is a safeguard for null value that breaks scenes dashboards.
+        //    expecting error at .includes(null) in order to not adjust
+        //    VariableOption type to avoid breaking changes
+        if (
+          variable.current.value === null ||
+          //@ts-expect-error
+          (Array.isArray(variable.current.value) && variable.current.value.includes(null))
+        ) {
+          variable.current = undefined;
+        }
+      }
+    }
+    return templating;
+  }
+
   private ensureListExist(data: any = {}) {
     data.list ??= [];
     return data;
@@ -1168,7 +1141,7 @@ export class DashboardModel implements TimeModel {
     let visibleHeight = viewHeight - navbarHeight - margin;
 
     // add back navbar height
-    if (kioskMode && kioskMode !== KioskMode.TV) {
+    if (kioskMode) {
       visibleHeight += navbarHeight;
     }
 
@@ -1216,7 +1189,14 @@ export class DashboardModel implements TimeModel {
   toggleExemplarsForAll() {
     for (const panel of this.panels) {
       for (const target of panel.targets) {
-        if (!(target.datasource && target.datasource.type === 'prometheus')) {
+        if (
+          !(
+            target.datasource &&
+            (target.datasource.type === 'prometheus' ||
+              target.datasource.type === 'grafana-amazonprometheus-datasource' ||
+              target.datasource.type === 'grafana-azureprometheus-datasource')
+          )
+        ) {
           continue;
         }
 
@@ -1354,8 +1334,10 @@ export class DashboardModel implements TimeModel {
     return this.panels.some((panel) => {
       // Return false for plugins that are angular but have angular.hideDeprecation = false
       // We cannot use panel.plugin.isAngularPlugin() because panel.plugin may not be initialized at this stage.
+      // We also have to check for old core angular plugins (explicitlyControlledMigrationPanels).
       const isAngularPanel =
-        config.panels[panel.type]?.angular?.detected && !config.panels[panel.type]?.angular?.hideDeprecation;
+        (config.panels[panel.type]?.angular?.detected || explicitlyControlledMigrationPanels.includes(panel.type)) &&
+        !config.panels[panel.type]?.angular?.hideDeprecation;
       let isAngularDs = false;
       if (panel.datasource?.uid) {
         isAngularDs = isAngularDatasourcePluginAndNotHidden(panel.datasource?.uid);
